import inspect
import json
from inspect import Signature
from typing import ClassVar, Any

from django.http import HttpRequest, HttpResponse, QueryDict
from django.views import View
from htpy import Node, render_node  # type: ignore


class HtpyView(View):
    # noinspection PyUnusedLocal
    def get(self, request, *args, **kwargs):
        return HttpResponse(render_node(self.render(request, *args, **kwargs)))

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def render(self, request: HttpRequest, *args, **kwargs) -> Node:
        raise NotImplementedError()


class HtPyActionsMixin:
    hp_action_params_meta: ClassVar[dict[str, Signature]]

    def _get_action_method_and_meta(self, action: str):
        method = getattr(self, f"action_{action}", None)
        if method is None:
            raise Exception(
                f"Unknown action: '{action}' ({self} doesn't implement the method 'action_{action}(request)')."
            )

        if not hasattr(self.__class__, "hp_action_params_meta"):
            self.__class__.hp_action_params_meta = {}

        meta = self.__class__.hp_action_params_meta
        if method.__name__ not in meta:
            meta[method.__name__] = inspect.signature(method)

        return method, meta[method.__name__]

    def action_attrs(self, name: str):
        """
        Validates and prepares parameters for an action method call.

        Parameters
        ----------
        name
            The name of the action method to be invoked.
        """
        self._get_action_method_and_meta(name)  # raises exception on missing action method

        return {
            "hx-patch": self.path,  # type: ignore
            **hx_vals(__hp_action=name),
        }

    def patch(self, request: HttpRequest, *args, **kwargs):
        qd = QueryDict(request.body, encoding=request.encoding)
        if (action := qd.get("__hp_action")) is not None:
            method, signature = self._get_action_method_and_meta(action)
            response = method() if len(signature.parameters) == 0 else method(request)
            if response is not None:
                return response

        return self.get(request, *args, **kwargs)  # type: ignore


def hx_vals(**kwargs) -> dict[str, Any]:
    return {"hx-vals": json.dumps({str(k): v for k, v in kwargs.items()})}
